@@ -50,17 +50,23 @@
 #include "pi-header.h"
 #include "pi-util.h"
 #include "qof-main.h"
+#include "palm.h"
 #include "pilot-qof.h"
 #include "pilot-todo.h"
 #include "qof-address.h"
 #include "qof-datebook.h"
-#include "qof-expenses.h"
+#ifdef HAVE_QOFEXPENSES
+#include <qof-expenses.h>
+#endif
+#include "pilot-expenses.h"
 
 /** used to print debug logs. */
 static QofLogModule log_module = PQ_MOD_CLI;
 /** pack routines for communication with pilot */
 static GList *pilot_modules = NULL;
 
+static const gchar *env_pilotport = "PILOTPORT";
+
 #define ARGUMENT_BAD_OPTION	17227
 
 #define EXCLUDE_REPEATER_SQL "SELECT * from pilot_datebook " \
@@ -109,9 +115,15 @@ pilot_qof_init (void)
 {
 	qof_init ();
 	g_return_val_if_fail (AddressRegister (), NULL);
+	/** @bug need to get Expenses from gpe-expenses and then
+	 add the pisock layer. */
+#ifdef HAVE_QOFEXPENSES
 	g_return_val_if_fail (ExpensesRegister (), NULL);
+#endif
+	g_return_val_if_fail (PQExpensesRegister (), NULL);
 	g_return_val_if_fail (DateBookRegister (), NULL);
 	g_return_val_if_fail (ToDoRegister (), NULL);
+	g_return_val_if_fail (packing_registration(), NULL);
 	return pilot_qof_create ();
 }
 
@@ -123,8 +135,6 @@ typedef enum {
 	PLU_CAT_WARN_UNKNOWN = 0x0008
 } plu_findcategory_flags_t;
 
-static const gchar *env_pilotport = "PILOTPORT";
-
 /* copied from pilot-link 0.12 source */
 static gint 
 plu_connect(gchar* plu_port, gint plu_quiet)
@@ -469,16 +479,19 @@ pilot_database_open (QofObject * obj, gpointer user_data)
 		return;
 	if (p->db_pref_unpack)
 	{
+		gchar * creator;
 		gint pref_result = 0;
 		/* Optionally, unpack preferences here */
 		pref_unpack = p->db_pref_unpack;
 		/* no entity available for the preferences, */
 		/* pass NULL and work only in the context. */
+		creator = g_strdup (p->pref_creator);
 		pref_result = dlp_ReadAppPreference (context->sd,
-			makelong (p->pref_creator), p->pref_flag,
+			makelong (creator), p->pref_flag,
 			PQ_PREF_USE_BACKUP, PQ_DEF_BUFSZ, context->pref_buf,
 			PQ_PREF_USE_SIZE, PQ_PREF_VERSION);
 		pref_unpack (NULL, context);
+		g_free (creator);
 	}
 	if (dlp_OpenDB (context->sd, PQ_DLP_CARD,
 			PI_DLP_ARG_FLAG_SHORT | PI_DLP_ARG_FLAG_LONG,
@@ -827,6 +840,8 @@ main (int argc, const char *argv[])
 				"<port>"},
 		{ "quiet",   'q', POPT_ARG_NONE,  &plu_quiet,  0 ,
 		 _("Suppress HotSync connection messages"), NULL},
+		{"input-file", 'i', POPT_ARG_STRING, &filename, qof_op_offline,
+		 _("Query the data in <filename>"), _("filename")},
 		QOF_CLI_OPTIONS 
 		{"hot-query", 'a', POPT_ARG_NONE,
 				NULL, qof_op_hotsync,
@@ -1137,11 +1152,23 @@ main (int argc, const char *argv[])
 			qof_cmd_hotsync (pilot_qof_context);
 			break;
 		}
+	case qof_op_noop:
+	case qof_op_category:
+	case qof_op_database:
+	case qof_op_time:
+	case qof_op_exclude:
+	case qof_op_sql:
+	case qof_op_sql_file:
+	case qof_op_write:
+	case qof_op_upload:
+	case qof_op_vers:
+	case qof_op_compress:
+	case qof_op_debug:
+	case qof_op_inv_city:
+	case qof_op_inv_vendor:
+	case qof_op_use_locale:
 	default:
-		{
-			/* should be impossible */
 			break;
-		}
 	}
 	poptFreeContext (pc);
 	pilot_qof_free (pilot_qof_context);
